using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;

namespace ThreadedGUI {
	public class ThreadedGUIForm : System.Windows.Forms.Form {
		private System.Windows.Forms.Button startSlowTaskButton;
		private System.Windows.Forms.Label counterLabel;
		private System.Windows.Forms.Label counter;
		private System.ComponentModel.Container components = null;

		int counterValue = 1;
		private System.Windows.Forms.ProgressBar progressBar;
		private System.Windows.Forms.Button threadedSlowTask;
		Thread counterThread = null;

		public ThreadedGUIForm() {
			InitializeComponent();

			this.Load += new EventHandler(Form1_Load);
			this.Deactivate += new EventHandler(Form1_Deactivate);
		}

		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.startSlowTaskButton = new System.Windows.Forms.Button();
			this.counterLabel = new System.Windows.Forms.Label();
			this.counter = new System.Windows.Forms.Label();
			this.progressBar = new System.Windows.Forms.ProgressBar();
			this.threadedSlowTask = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// startSlowTaskButton
			// 
			this.startSlowTaskButton.Location = new System.Drawing.Point(32, 112);
			this.startSlowTaskButton.Name = "startSlowTaskButton";
			this.startSlowTaskButton.Size = new System.Drawing.Size(112, 23);
			this.startSlowTaskButton.TabIndex = 0;
			this.startSlowTaskButton.Text = "Start Slow Task";
			this.startSlowTaskButton.Click += new System.EventHandler(this.startSlowTaskButton_Click);
			// 
			// counterLabel
			// 
			this.counterLabel.Location = new System.Drawing.Point(32, 56);
			this.counterLabel.Name = "counterLabel";
			this.counterLabel.Size = new System.Drawing.Size(56, 23);
			this.counterLabel.TabIndex = 1;
			this.counterLabel.Text = "Counter:";
			// 
			// counter
			// 
			this.counter.Location = new System.Drawing.Point(96, 56);
			this.counter.Name = "counter";
			this.counter.Size = new System.Drawing.Size(48, 23);
			this.counter.TabIndex = 2;
			// 
			// progressBar
			// 
			this.progressBar.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.progressBar.Location = new System.Drawing.Point(0, 250);
			this.progressBar.Name = "progressBar";
			this.progressBar.Size = new System.Drawing.Size(292, 23);
			this.progressBar.TabIndex = 3;
			// 
			// threadedSlowTask
			// 
			this.threadedSlowTask.Location = new System.Drawing.Point(32, 152);
			this.threadedSlowTask.Name = "threadedSlowTask";
			this.threadedSlowTask.Size = new System.Drawing.Size(152, 23);
			this.threadedSlowTask.TabIndex = 4;
			this.threadedSlowTask.Text = "Start Threaded Slow Task";
			this.threadedSlowTask.Click += new System.EventHandler(this.threadedSlowTask_Click);
			// 
			// ThreadedGUIForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Controls.Add(this.threadedSlowTask);
			this.Controls.Add(this.progressBar);
			this.Controls.Add(this.counter);
			this.Controls.Add(this.counterLabel);
			this.Controls.Add(this.startSlowTaskButton);
			this.Name = "ThreadedGUIForm";
			this.Text = "Threaded GUI Tester";
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new ThreadedGUIForm());
		}

		private void Form1_Load(object sender, EventArgs e) {
			// Don't start thread until form is loaded because can't call Invoke until controls are displayed
			counterThread = new Thread(new ThreadStart(IncrementCounter));
			counterThread.Start();
		}

		private void Form1_Deactivate(object sender, EventArgs e) {
			// Stop counter thread when form goes away
			counterThread.Abort();
		}

		// This gets called in a special thread
		protected void IncrementCounter() {
			do {
				BeginInvoke(new MethodInvoker(UpdateCounterDisplay));  // Force update to happen in event thread
				counterValue++;
				Thread.Sleep(100);
			} while (true);
		}

		protected void UpdateCounterDisplay() {
			counter.Text = counterValue.ToString();
		}

		// UNTHREADED SLOW TASK
		private void startSlowTaskButton_Click(object sender, System.EventArgs e) {
			progressBar.Minimum = 1;
			progressBar.Maximum = 10;

			for (int i=1; i<=10; i++) {
				Thread.Sleep(1000);
				progressBar.Value = i;
			}
		}

		////// STUFF FOR THREADED SLOW TASK
		private void threadedSlowTask_Click(object sender, System.EventArgs e) {
			progressBar.Minimum = 1;
			progressBar.Maximum = 10;

			Thread slowTaskThread = new Thread(new ThreadStart(SlowTaskThread));
			slowTaskThread.Start();
		}

		private void SlowTaskThread() {
			for (int i=1; i<=10; i++) {
				Thread.Sleep(1000);
				BeginInvoke(new IntInvoker(SetProgressBar), new object[] {i});
				progressBar.Value = i;
			}		
		}

		delegate void IntInvoker(int i);

		private void SetProgressBar(int i) {
			progressBar.Value = i;
		}
	}
}